local seed = os.time()
local sin = math.sin
local random = math.random
local twopi = math.pi * 2

local scale = 4
local n = 5
local time = 0
local img_globe, img_jar, img_player
local n_jar, n_player
local canvas
local effect

local function oscillate(amplitude, freq, phase)
	return amplitude * sin(freq * twopi * time + phase)
end

local lights = {
	{
		pos = {x = 0, y = 0, z = 0},
		diffuse = {1, 0, 0},
	},
	{
		pos = {x = 0, y = 0, z = 0},
		diffuse = {0, 1, 0},
	},
	{
		pos = {x = 0, y = 0, z = 0},
		diffuse = {0, 0, 1},
	},
	{
		pos = {x = 0, y = 0, z = 0},
		diffuse = {1, 1, 0},
	},
	{
		pos = {x = 0, y = 0, z = 0},
		diffuse = {0, 1, 1},
	},
}

math.randomseed(seed)

local function send_light(light)
	local pos = light.pos

	effect:send("light_position", {pos.x, pos.y, pos.z})
	effect:send("light_diffuse", {unpack(light.diffuse)})
end

local function draw_lights()
	love.graphics.setShader()
	love.graphics.setBlendMode("add")

	for i = 1, n do
		local light = lights[i]
		local pos = light.pos
		local r, g, b = unpack(light.diffuse)
		local w, h = img_globe:getDimensions()
		local ox = w/2
		local oy = h/2
		local x = pos.x
		local y = love.graphics.getHeight()/scale - pos.y
		local z = pos.z

		love.graphics.setColor(r, g, b)
		-- love.graphics.circle("fill", x, y, 8)
		love.graphics.draw(img_globe, x, y - z, 0, 1, 1, ox, oy)
	end

	love.graphics.setBlendMode("alpha")
end

function love.load()
	love.graphics.setBackgroundColor(0.3, 0.3, 0.5)
	love.graphics.setDefaultFilter("nearest", "nearest")

	img_globe = love.graphics.newImage("globe.png")
	img_jar = love.graphics.newImage("jar.png")
	n_jar = love.graphics.newImage("n_jar.png")
	img_player = love.graphics.newImage("player.png")
	n_player = love.graphics.newImage("n_player.png")

	canvas = love.graphics.newCanvas(love.graphics.getWidth()/scale, love.graphics.getHeight()/scale)
	canvas:setFilter("nearest", "nearest")

	effect = love.graphics.newShader("multilights.glsl")
end

function love.update(dt)
	time = time + dt
	local z = 20 + oscillate(15, 0.1, 0)

	local mx, my = love.mouse.getPosition()
	local mouse_light = lights[1]
	local m_pos = mouse_light.pos

	m_pos.x = mx/scale
	m_pos.y = my/scale
	m_pos.z = z
	send_light(mouse_light)

	-- math.randomseed(seed)
    --
	-- for i = 2, n do
	-- 	local light = lights[i]
	-- 	local pos = light.pos
	-- 	local w, h = love.graphics.getDimensions()
    --
	-- 	pos.x = w/scale/2 + oscillate(50, 41/400, random() * 5)
	-- 	pos.y = h/scale/2 + oscillate(50, 47/400, random() * 5)
	-- 	pos.z = z
    --
	-- 	send_light(light)
	-- end
end

function love.draw()
	love.graphics.setCanvas(canvas)
	love.graphics.clear()
	love.graphics.setShader()
	love.graphics.setColor(220/255, 190/255, 0)
	love.graphics.setShader(effect)
	love.graphics.setColor(1, 1, 1)

	local w, h = love.graphics.getDimensions()
	w = w/scale
	h = h/scale

	local jw, jh = img_jar:getDimensions()
	local pw, ph = img_player:getDimensions()

	love.graphics.draw(img_jar, w/2 - 32, h/2, 0, 1, 1, jw/2, jh/2)
	effect:send("normal_texture", n_jar)

	love.graphics.draw(img_player, w/2 + 32, h/2, 0, 1, 1, pw/2, ph/2)
	effect:send("normal_texture", n_player)

	love.graphics.setShader()
	draw_lights()

	love.graphics.setCanvas()
	love.graphics.setShader()
	love.graphics.setColor(1, 1, 1)
	love.graphics.draw(canvas, 0, 0, 0, scale, scale)
end
