#define MAX_LIGHTS 5

extern Image normal_texture;
extern vec3 light_position[MAX_LIGHTS];
extern vec3 light_diffuse[MAX_LIGHTS];

extern vec3 dark_color = vec3(0.0, 0.0, 1.0);
extern vec3 light_color = vec3(0.25, 0.25, 0.0);

float luminance(vec3 light)
{
	return dot(light, vec3(0.2126, 0.7152, 0.0722));
}

vec4 effect(vec4 color, Image tex, vec2 tc, vec2 pc)
{
	vec4 diffuse = pow(Texel(tex, tc), vec4(2.2));
	vec4 channels = Texel(tex, tc + vec2(0.66667, 0));
	float ao = channels.r;
	float bypass = channels.g;
	vec3 normal = Texel(normal_texture, tc).rgb;

	normal.y = 1 - normal.y;
	normal = normalize(mix(vec3(-1), vec3(1), normal));

	vec3 light = vec3(0);
	vec3 cel_light = vec3(0);

	for (int i = 0; i < light_position.length(); i++)
	{
		vec3 light_dir = light_position[i] - vec3(pc, 0);
		float dist = length(light_dir);
		float attenuation = smoothstep(100, 50, dist);

		light_dir = normalize(light_dir);

		vec3 current_light = clamp(dot(normal, light_dir), 0.0, 1.0)
			* light_diffuse[i] * attenuation * (ao * 0.7 + 0.3);

		light += current_light;

		float n = (current_light.r + current_light.g + current_light.b)/3.0;
		cel_light += step(0.2, n) * current_light * diffuse.rgb;
	}

	vec3 gooch_light = mix(dark_color, light_color, luminance(light)) * 0.06;
	vec3 ambient = diffuse.rgb * 0.06;
	vec3 final_light = pow(gooch_light + ambient + cel_light, vec3(0.4545));

	return vec4(mix(diffuse.rgb, final_light, bypass), diffuse.a);
}
