extern Image normal_texture;
extern vec3 light_position[5];
extern vec3 light_diffuse[5];

// calculate luminance from linear rgb
float luminance(vec3 light) {
   return dot(light, vec3(0.2126, 0.7152, 0.0722));
}

vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords) {
   vec4 diffuse = pow(Texel(texture, texture_coords), vec4(2.2));
   vec4 channels = Texel(texture, texture_coords + vec2(0.66667, 0));
   float ao = channels.r;
   float bypass = channels.g;

    //vec3 normal = Texel(texture, texture_coords + vec2(0.3333, 0)).rgb;
   vec3 normal = Texel(normal_texture, texture_coords).rgb;
   normal.y = 1 - normal.y;
   normal = normalize(mix(vec3(-1), vec3(1), normal));

   vec3 light = vec3(0);
   vec3 cel_light = vec3(0);
   for (int i = 0; i < light_position.length(); i++) {
      vec3 light_direction = light_position[i] - vec3(pixel_coords, 0);
      float dist = length(light_direction);
      float attenuation = smoothstep(100, 50, dist);
      light_direction = normalize(light_direction);

      vec3 current_light = clamp(dot(normal, light_direction), 0.0, 1.0) * light_diffuse[i] * attenuation * (ao * 0.7 + 0.3);

      light += current_light;

      cel_light += step(0.2, (current_light.r + current_light.g + current_light.b) / 3.0) * current_light * diffuse.rgb;
   }

   vec3 dark_color = vec3(0.0, 0.0, 1.0);
   vec3 light_color = vec3(0.25, 0.25, 0.0);
   //vec3 gooch_light = mix(dark_color, light_color, luminance(light)) * 0.25 * diffuse.rgb; // not sure if diffuse should be included here
   vec3 gooch_light = mix(dark_color, light_color, luminance(light)) * 0.06;
   vec3 ambient = diffuse.rgb * 0.06;

   vec3 final_light = pow(gooch_light + ambient + cel_light, vec3(0.4545));
   return vec4(mix(diffuse.rgb, final_light, bypass), diffuse.a);
}
